@@ -133,7 +133,7 @@ def slitfunc(img, ycen, lambda_sp=0, lambda_sf=0.1, osample=1):
     return sp, sl, model, unc, mask
 
 
-def slitfunc_curved(img, ycen, tilt, shear, lambda_sp, lambda_sf, osample, yrange):
+def slitfunc_curved(img, ycen, tilt, shear, lambda_sp, lambda_sf, osample, yrange, maxiter=20):
     """Decompose an image into a spectrum and a slitfunction, image may be curved
 
     Parameters
@@ -166,12 +166,14 @@ def slitfunc_curved(img, ycen, tilt, shear, lambda_sp, lambda_sf, osample, yrang
     lambda_sf = float(lambda_sf)
     lambda_sp = float(lambda_sp)
     osample = int(osample)
+    maxiter = int(maxiter)
     img = np.asanyarray(img, dtype=c_double)
     ycen = np.asarray(ycen, dtype=c_double)
     yrange = np.asarray(yrange, dtype=int)
 
     assert img.ndim == 2, "Image must be 2 dimensional"
     assert ycen.ndim == 1, "Ycen must be 1 dimensional"
+    assert maxiter > 0, "Maximum iterations must be positive"
 
     if np.isscalar(tilt):
         tilt = np.full(img.shape[1], tilt, dtype=c_double)
@@ -267,7 +269,10 @@ def slitfunc_curved(img, ycen, tilt, shear, lambda_sp, lambda_sf, osample, yrang
     # Info contains the folowing: sucess, cost, status, iteration, delta_x
     info = np.zeros(5, dtype=c_double)
 
-    assert not np.any(np.sum(mask, axis=0) == 0), "At least one mask column is all 0."
+    col = np.sum(mask, axis=0) == 0
+    if np.any(col):
+       mask[mask.shape[0] // 2, col] = 1
+    # assert not np.any(np.sum(mask, axis=0) == 0), "At least one mask column is all 0."
 
     # Call the C function
     slitfunc_2dlib.slit_func_curved(
@@ -283,6 +288,7 @@ def slitfunc_curved(img, ycen, tilt, shear, lambda_sp, lambda_sf, osample, yrang
         ffi.cast("int", osample),
         ffi.cast("double", lambda_sp),
         ffi.cast("double", lambda_sf),
+        ffi.cast("int", maxiter),
         ffi.cast("double *", psf_curve.ctypes.data),
         ffi.cast("double *", sp.ctypes.data),
         ffi.cast("double *", sl.ctypes.data),
@@ -300,7 +306,7 @@ def slitfunc_curved(img, ycen, tilt, shear, lambda_sp, lambda_sf, osample, yrang
         if status == 0:
             msg = "I dont't know what happened"
         elif status == -1:
-            msg = "Did not finish convergence after maxiter iterations"
+            msg = f"Did not finish convergence after maxiter ({maxiter}) iterations"
         elif status == -2:
             msg = "Curvature is larger than the swath. Check the curvature!"
         else:
