@@ -17,6 +17,7 @@ from scipy.constants import speed_of_light
 from scipy.interpolate import interp1d
 from scipy.optimize import curve_fit
 from scipy.ndimage.morphology import grey_closing
+from scipy.ndimage.filters import gaussian_filter1d
 
 from astropy.io import fits
 
@@ -132,15 +133,32 @@ class LineAtlas:
         self.fname = join(folder, "wavecal/atlas", fname)
         self.wave, self.flux = self.load_fits(self.fname)
 
-        fname_list = element.lower() + "_list.txt"
-        self.fname_list = join(folder, "wavecal/atlas", fname_list)
-        linelist = np.genfromtxt(self.fname_list, dtype="f8,U8")
-        wpos, element = linelist["f0"], linelist["f1"]
-        indices = self.wave.searchsorted(wpos)
-        heights = self.flux[indices]
-        self.linelist = np.rec.fromarrays(
-            [wpos, heights, element], names=["wave", "heights", "element"]
-        )
+        try:
+            # If a specific linelist file is provided
+            fname_list = element.lower() + "_list.txt"
+            self.fname_list = join(folder, "wavecal/atlas", fname_list)
+            linelist = np.genfromtxt(self.fname_list, dtype="f8,U8")
+            wpos, element = linelist["f0"], linelist["f1"]
+            indices = self.wave.searchsorted(wpos)
+            heights = self.flux[indices]
+            self.linelist = np.rec.fromarrays(
+                [wpos, heights, element], names=["wave", "heights", "element"]
+            )
+        except (FileNotFoundError, IOError):
+            # Otherwise fit the line positions from the spectrum
+            logger.warning(
+                "No dedicated linelist found for %s, determining peaks based on the reference spectrum instead.",
+                element,
+            )
+            module = WavelengthCalibration(plot=False)
+            n, peaks = module._find_peaks(self.flux)
+            wpos = np.interp(peaks, np.arange(len(self.wave)), self.wave)
+            element = np.full(len(wpos), element)
+            indices = self.wave.searchsorted(wpos)
+            heights = self.flux[indices]
+            self.linelist = np.rec.fromarrays(
+                [wpos, heights, element], names=["wave", "heights", "element"]
+            )
 
         # The data files are in vaccuum, if the instrument is in air, we need to convert
         if medium == "air":
@@ -149,15 +167,24 @@ class LineAtlas:
 
     def load_fits(self, fname):
         hdu = fits.open(fname)
-        header = hdu[0].header
-        data = hdu[0].data.ravel()
-        data /= data.max()
-        data = np.clip(data, 0, None)
+        if len(hdu) == 1:
+            # Its just the spectrum
+            # with the wavelength defined via the header keywords
+            header = hdu[0].header
+            spec = hdu[0].data.ravel()
+            wmin = header["CRVAL1"]
+            wdel = header["CDELT1"]
+            wave = np.arange(spec.size) * wdel + wmin
+        else:
+            # Its a binary Table, with two columns for the wavelength and the
+            # spectrum
+            data = hdu[1].data
+            wave = data["wave"]
+            spec = data["spec"]
 
-        wmin = header["CRVAL1"]
-        wdel = header["CDELT1"]
-        wave = np.arange(data.size) * wdel + wmin
-        return wave, data
+        spec /= np.nanmax(spec)
+        spec = np.clip(spec, 0, None)
+        return wave, spec
 
 
 class LineList:
@@ -331,14 +358,18 @@ class WavelengthCalibration:
             if self.closing > 0:
                 obs[i] = grey_closing(obs[i], self.closing)
             try:
-                obs[i] -= np.ma.min(obs[i][obs[i] > 0])
+                obs[i] -= np.ma.median(obs[i][obs[i] > 0])
             except ValueError:
                 logger.warning(
                     f"Could not determine the minimum value in order %i. No positive values found",
                     i,
                 )
             obs[i] /= np.ma.max(obs[i])
-        obs[obs <= 0] = np.ma.masked
+        
+        # Remove negative outliers
+        std = np.std(obs, axis=1)[:, None]
+        obs[obs <= -2 * std] = np.ma.masked
+        # obs[obs <= 0] = np.ma.masked
 
         # Normalize lines in each order
         for order in np.unique(lines["order"]):
@@ -1211,8 +1242,11 @@ class WavelengthCalibration:
         for j, p in enumerate(peaks):
             idx = p + np.arange(-width, width + 1, 1)
             idx = np.clip(idx, 0, len(c) - 1).astype(int)
-            coef = util.gaussfit3(np.arange(len(idx)), c[idx])
-            new_peaks[j] = coef[1] + p - width
+            try:
+                coef = util.gaussfit3(np.arange(len(idx)), c[idx])
+                new_peaks[j] = coef[1] + p - width
+            except RuntimeError:
+                new_peaks[j] = p
 
         n = np.arange(len(peaks))
 
@@ -1356,33 +1390,19 @@ class WavelengthCalibrationComb(WavelengthCalibration):
             # Use the existing absolute wavelength calibration as reference
             y_ord = np.full(len(peaks), i)
             w_old = interp1d(np.arange(len(wave[i])), wave[i], kind="cubic")(peaks)
-            # w_old = np.interp(peaks, np.arange(len(wave[i])), wave[i])
-            # w_old = self.evaluate_solution(peaks, y_ord, wave_solution)
             f_old = speed_of_light / w_old
 
             # fr: repeating frequency
             # fd: anchor frequency of this order, needs to be shifted to the absolute reference frame
-            # res = Polynomial.fit(n, f_old, deg=1, domain=[])
-            # fd, fr = res.coef
-
             fr = np.median(np.diff(f_old))
             fd = np.median(f_old % fr)
             n_raw = (f_old - fd) / fr
             n = np.round(n_raw)
 
             if np.any(np.abs(n_raw - n) > 0.3):
-                logger.warning("Bad peaks detected in the frequency comb")
+                logger.warning("Bad peaks detected in the frequency comb in order %i", i)
 
-            # res = Polynomial.fit(n, f_old, deg=1, domain=[])
             fr, fd = polyfit(n, f_old, deg=1)
-            # f_old = np.polyval([fr, fd], n)
-
-            # w_old = speed_of_light / f_old
-            # m = np.zeros_like(w_old)
-            # m[1:] = w_old[1:] / np.diff(w_old)
-            # m = np.abs(np.round(m))
-            # m[0] = 1 + m[1]
-            # n = m
 
             n_offset = 0
             # The first order is used as the baseline for all other orders
@@ -1413,38 +1433,27 @@ class WavelengthCalibrationComb(WavelengthCalibration):
         # where m is the peak number
         # this is the result of the grating equation
         # at least const is roughly constant for neighbouring peaks
+        correct = True
+        if correct: 
+            w_all = [speed_of_light / f for f in f_all]
+            mw_all = [m * w for m, w in zip(n_all, w_all)]
+            y = np.concatenate(mw_all)
+            gap = np.median(y)
 
-        w_all = [speed_of_light / f for f in f_all]
-        mw_all = [m * w for m, w in zip(n_all, w_all)]
-        y = np.concatenate(mw_all)
-        gap = np.median(y)
+            corr = np.zeros(self.nord)
+            for i in range(self.nord):
+                corri = gap / w_all[i] - n_all[i]
+                corri = np.median(corri)
+                corr[i] = np.round(corri)
+                n_all[i] += corr[i]
 
-        # for i in range(len(n_all)):
-        #     plt.plot(n_all[i], n_all[i] * w_all[i])
-        # plt.show()
+            logger.debug("LFC order offset correction: %s", corr)
 
-        corr = np.zeros(self.nord)
-        for i in range(self.nord):
-            corri = gap / w_all[i] - n_all[i]
-            corri = np.median(corri)
-            corr[i] = np.round(corri)
-            n_all[i] += corr[i]
-
-        logger.debug("LFC order offset correction: %s", corr)
-
-        # for i in range(len(n_all)):
-        #     plt.plot(n_all[i], n_all[i] * w_all[i])
-        # plt.show()
-
-        for i in range(self.nord):
-            coef = polyfit(n_all[i], n_all[i] * w_all[i], deg=5)
-            mw = np.polyval(coef, n_all[i])
-            w_all[i] = mw / n_all[i]
-            f_all[i] = speed_of_light / w_all[i]
-
-        # for i in range(len(n_all)):
-        #     plt.plot(n_all[i], n_all[i] * w_all[i])
-        # plt.show()
+            for i in range(self.nord):
+                coef = polyfit(n_all[i], n_all[i] * w_all[i], deg=5)
+                mw = np.polyval(coef, n_all[i])
+                w_all[i] = mw / n_all[i]
+                f_all[i] = speed_of_light / w_all[i]
 
         # Merge Data
         n_all = np.concatenate(n_all)
@@ -1533,12 +1542,14 @@ class WavelengthCalibrationInitialize(WavelengthCalibration):
         degree=2,
         plot=False,
         plot_title="Wavecal Initial",
-        wave_delta = 20,
-        nwalkers = 100,
-        steps = 20_000,
-        resid_delta = 1000,
-        element = "thar",
-        medium = "vac",
+        wave_delta=20,
+        nwalkers=100,
+        steps=50_000,
+        resid_delta=1000,
+        cutoff=5,
+        smoothing=0,
+        element="thar",
+        medium="vac",
     ):
         super().__init__(
             degree=degree,
@@ -1548,17 +1559,40 @@ class WavelengthCalibrationInitialize(WavelengthCalibration):
             plot_title=plot_title,
             dimensionality="1D",
         )
+        #:float: wavelength uncertainty on the initial guess in Angstrom
         self.wave_delta = wave_delta
+        #:int: number of walkers in the MCMC
         self.nwalkers = nwalkers
+        #:int: number of steps in the MCMC
         self.steps = steps
+        #:float: residual uncertainty allowed when matching observation with known lines
         self.resid_delta = resid_delta
+        #:float: gaussian smoothing applied to the wavecal spectrum before the MCMC in pixel scale, disable it by setting it to 0
+        self.smoothing = smoothing
+        #:float: minimum value in the spectrum to be considered a spectral line, if the value is above (or equal 1) it defines the percentile of the spectrum
+        self.cutoff = cutoff
 
+    def get_cutoff(self, spectrum):
+        if self.cutoff == 0:
+            cutoff = None
+        elif self.cutoff < 1:
+            cutoff = self.cutoff
+        else:
+            cutoff = np.nanpercentile(spectrum[spectrum != 0], self.cutoff)
+        return cutoff
+
+    def normalize(self, spectrum):
+        smoothing = self.smoothing
+        spectrum = np.copy(spectrum)
+        spectrum -= np.nanmedian(spectrum)
+        if smoothing != 0:
+            spectrum = gaussian_filter1d(spectrum, smoothing)
+        spectrum[spectrum < 0] = 0
+        spectrum /= np.max(spectrum)
+        return spectrum
 
     def determine_wavelength_coefficients(
-        self,
-        spectrum,
-        atlas,
-        wave_range,
+        self, spectrum, atlas, wave_range,
     ) -> np.ndarray:
         """
         Determines the wavelength polynomial coefficients of a spectrum, 
@@ -1604,8 +1638,9 @@ class WavelengthCalibrationInitialize(WavelengthCalibration):
         n_output = ndim = self.degree + 1
 
         # Normalize the spectrum, and copy it just in case
-        spectrum = spectrum / np.max(spectrum)
-
+        spectrum = self.normalize(spectrum)
+        cutoff = self.get_cutoff(spectrum)
+       
         # The pixel scale used for everything else
         x = np.arange(n_features)
         # Initial guess for the wavelength solution
@@ -1645,6 +1680,8 @@ class WavelengthCalibrationInitialize(WavelengthCalibration):
             # Chech that w is increasing
             prior = np.zeros(w.shape[0])
             prior[np.any(w[:, 1:] < w[:, :-1], axis=1)] = -np.inf
+            prior[w[:, 0] < wave_range[0] - self.wave_delta] = -np.inf
+            prior[w[:, -1] > wave_range[1] + self.wave_delta] = -np.inf
             return prior
 
         def log_prob(p):
@@ -1662,12 +1699,21 @@ class WavelengthCalibrationInitialize(WavelengthCalibration):
             y[where, :] /= np.max(y[where, :], axis=1)[:, None]
             # This is the cross correlation value squared
             cross = np.sum(y * spectrum, axis=1) ** 2
-            # chi2 = - np.sum((y - spectrum)**2)
-            return prior + cross
+            # chi2 = - np.sum((y - spectrum)**2, axis=1)
+            # chi2 = - np.sum((np.where(y > 0.01, 1, 0) - np.where(spectrum > 0.01, 1, 0))**2, axis=1)
+            # this is the same as above, but a lot faster thanks to the magic of bitwise xor
+            if cutoff is not None:
+                chi2 = (y > cutoff) ^ (spectrum > cutoff)
+                chi2 = -np.count_nonzero(chi2, axis=1) / 20
+            else:
+                chi2 = -np.sum((y - spectrum) ** 2, axis=1) / 20
+            return prior + cross + chi2
 
         p0 = np.zeros((self.nwalkers, ndim))
         p0 += coef[None, :]
-        p0 += np.random.uniform(low=-self.wave_delta, high=self.wave_delta, size=(self.nwalkers, ndim))
+        p0 += np.random.uniform(
+            low=-self.wave_delta, high=self.wave_delta, size=(self.nwalkers, ndim)
+        )
         sampler = emcee.EnsembleSampler(
             self.nwalkers,
             ndim,
@@ -1742,13 +1788,16 @@ class WavelengthCalibrationInitialize(WavelengthCalibration):
 
         n_features = spectrum.shape[0]
         x = np.arange(n_features)
+        smoothing = self.smoothing
 
         # Normalize just in case
-        spectrum = spectrum / np.max(spectrum)
+        spectrum = self.normalize(spectrum)
+        cutoff = self.get_cutoff(spectrum)
 
         # TODO: make this use another function, and pass the hight as a parameter
         scopy = np.copy(spectrum)
-        scopy[scopy < 0.01] = 0
+        if cutoff is not None:
+            scopy[scopy < cutoff] = 0
         _, peaks = self._find_peaks(scopy)
 
         peak_wave = np.interp(peaks, x, wavelength)
